==> GeneR_2.6.0.tar.gz: GeneR/src/translate.cc <==
/*! \file translate.cc
 *  \brief conversion nucleic => protein
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-02-16 16:30:13 lucas>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include "GeneR_glob.h"
#include <string>
#include <R.h>
#include <Rdefines.h>
#include <math.h>
#include "libIndex.h"

/* Global variables */

#define NBTABLES 5
/* Get from :
 *   http://embryology.med.unsw.edu.au/DNA/Genetic_Codes2.htm
 * 0: std code
 * 1: Vertebrate Mitochondrial Code
 * 2: Yeast Mitochondrial Code
 * 3: Mold, Protozoan, and Coelenterate Mitochondrial Code 
 *    and the Mycoplasma/Spiroplasma Code
 * 4: Invertebrate Mitochondrial Code
 */

/** \var codingtable 
 * \brief This is a list of character string. Each character string
 * must be of size 65, i.e. one letter for each tri-nucleotides +
 * a sign used when a letter non nucleotide (N) is in a tri-nucleotides.
 */
char * codingtable [NBTABLES] = { \
"FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSS**VVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWTTTTPPPPHHQQR--RIIMMTTTTNNKKSSRRVVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSSSSVVVVAAAADDEEGGGG-"};

extern "C"
{
  void strtranslateR (char ** nuc, char ** prot, int * quelle_table,
		      char **  chars_table);

  SEXP print_code_table (SEXP quelle_tableR, SEXP chars_tableR);
  
  SEXP translateR (SEXP bufnoR,SEXP fromR, SEXP toR, SEXP strandR, SEXP QuelleTableR, SEXP chars_tableR);

}


/** \brief translate a charater string like "FFLLSSSSYY**CC*WLLLLPPPP..."
 * of size 65 characters to a human reading table like TTT -> F.
 *
 * Conversion table:
 * All in the order:
 * 0   1   2   3   4
 * TTT TTC TTA TTG TCT... 
 * F   F   L   L   S 
 * Letters in this order T=0,C=1,A=2,G=3
 * 
 * In R:
 *  .C("strnuctoprotR","TTTTTCTTATTGTCT","            ")
 */

void strtranslateR (char ** nuc, char ** prot, int * quelle_table,
		    char **  chars_table)
{
  int i=0,j=0,k,nameprot;
  int longnuc;
  char * table;
  extern char * codingtable [NBTABLES];

  longnuc = strlen(*nuc);

  /* Choose coding table */
  table = codingtable[0];
  if((quelle_table[0] > 0) && (quelle_table[0] <  NBTABLES) ) 
    table= codingtable[quelle_table[0]];

  if(strlen(*chars_table)==65)
    table = * chars_table;


  /* while( (*nuc) [i] && (*nuc) [i+1] && (*nuc) [i+2])*/
  while( i < longnuc-2)
    {

      nameprot = 0;

      for(k = 2;k>=0; k--)
	{
	  switch((*nuc)[i++])
	    {
	    case 'T': break;
	    case 'U': break;
	    case 't': break;
	    case 'u': break;
	    case 'C': nameprot = nameprot + (int)pow(4,k); break;
	    case 'c': nameprot = nameprot + (int)pow(4,k); break;
	    case 'A': nameprot = nameprot + 2*(int)pow(4,k); break;
	    case 'a': nameprot = nameprot + 2*(int)pow(4,k); break;
	    case 'G': nameprot = nameprot + 3*(int)pow(4,k); break;
	    case 'g': nameprot = nameprot + 3*(int)pow(4,k); break;
	    default:  nameprot = 64; 
	    }
	}
      if(nameprot > 64 ) nameprot = 64;
      (*prot)[j++] = table[nameprot];
    }
  (*prot)[j]= '\0';
}


SEXP print_code_table (SEXP quelle_tableR, SEXP chars_tableR)
{
  SEXP ans;
  int i,j,k,l;
  char * table;
  int * quelle_tableC;
  char * chars_tableC;
  char  nucl[5] = "UCAG";
  char  nucl_tmp[4];
  char  prot_tmp[2];
  extern char * codingtable [NBTABLES];

  /* Import from R input */
  quelle_tableC = INTEGER(quelle_tableR);
  chars_tableC = CHAR(STRING_ELT(chars_tableR,0));
	  
  /* Choose coding table */
  table = codingtable[0];
  if((quelle_tableC[0] > 0) && (quelle_tableC[0] <  NBTABLES) ) 
    table= codingtable[quelle_tableC[0]];

  if((strlen(chars_tableC)==65) || (strlen(chars_tableC)==64))
    table = chars_tableC;
  

  /* Allocation of result into R */
  PROTECT(ans = allocVector(STRSXP, 128));
  l=0;
  nucl_tmp[3]='\0';
  for(i =0; i<4; i++)
    {
      nucl_tmp[0]= nucl[i];
      for(j =0; j<4; j++)
	{
	  nucl_tmp[1]= nucl[j];
	  for(k =0; k<4; k++)
	    {
	      nucl_tmp[2]= nucl[k];
	      SET_STRING_ELT(ans, l++, mkChar(nucl_tmp));
	    }
	}
    }
  
  prot_tmp[1]='\0';
  for (i = 0 ; i<64; i++)
    {
      prot_tmp[0] = table[i];
      SET_STRING_ELT(ans, l++, mkChar(prot_tmp));
    }

  UNPROTECT(1);
  return(ans);
}


/** Function Translate
 * \brief translate nucleotide into protein
 */
SEXP translateR (SEXP bufnoR,SEXP fromR, SEXP toR, SEXP strandR, SEXP QuelleTableR, SEXP chars_tableR)
{
  SEXP ans;
  int * fromC;
  int * toC;
  int * strandC;
  int  bufnoC;
  int * quelle_tableC;
  char * proteine;
  char * nucleotides;
  char * Buffie;
  char * chars_tableC;

  int nbregions,i,sizealloc,longbufno,onestrand=1,longueur;

  /* Get inputs from R */
  bufnoC = INTEGER(bufnoR)[0];
  fromC = INTEGER(fromR);
  toC = INTEGER(toR);
  strandC = INTEGER(strandR);
  quelle_tableC = INTEGER(QuelleTableR);
  nbregions =  LENGTH(fromR);
  chars_tableC = CHAR(STRING_ELT(chars_tableR,0));
  
  /* 2 cas of error */
  if((LENGTH(toR) != nbregions) )
    return(libIndex::returnInteger(-1));

  Buffie = myGeneRbuffer(bufnoC);
  if (Buffie == NULL)
    return(libIndex::returnInteger(-1));

  longbufno =   strlen(Buffie);

  /* allocation of temporary proteine */
  sizealloc = 0;
  for (i=0; i < nbregions; i++)
    {
      if(toC[i] == 0)
	toC[i] = longbufno;
      if((toC[i] - fromC[i] + 1) >sizealloc )
	sizealloc =toC[i] - fromC[i] + 1;
    }
  proteine = (char *) malloc( ((int) (sizealloc+2)/3 )* sizeof(char));
  nucleotides = (char *) malloc((sizealloc+1) * sizeof(char));
  
  /*  printf("Allocation faite nuc %d prot %d\n",sizealloc+1,((int) (sizealloc+2)/3 )); */

  if(LENGTH(strandR) ==nbregions)
    onestrand = nbregions+1;

   /* Allocation of result into R */
  PROTECT(ans = allocVector(STRSXP, nbregions));
  
  for (i=0; i < nbregions; i++)
    {
      Buffie = myGeneRbuffer(bufnoC,strandC[i%onestrand]);
    
      longueur = toC[i]-fromC[i]+1;
      if(longueur<0)
	longueur=1;
      nucleotides = strncpy(nucleotides,Buffie+fromC[i]-1,longueur);
      nucleotides[longueur] = '\0';
      /*  printf("Ds la boucle %d %d %s\n",strlen(nucleotides),fromC[i],nucleotides); */
  
      strtranslateR(&nucleotides,&proteine,quelle_tableC, &chars_tableC);

      /* Return to R */
      SET_STRING_ELT(ans, i, mkChar(proteine));
    }

  UNPROTECT(1);
  free(proteine);
  free(nucleotides);
  return(ans);
}

==> GeneR_2.8.0.tar.gz: GeneR/src/translate.cc <==
/*! \file translate.cc
 *  \brief conversion nucleic => protein
 *  \version 1
 *  \date Created       : 26/10/05 
 *  \date Last Modified : Time-stamp: <2006-02-16 16:30:13 lucas>
 *  \author A. Lucas
 *  \note Licence: CeCIll
 */

#include "GeneR_glob.h"
#include <string>
#include <R.h>
#include <Rdefines.h>
#include <math.h>
#include "libIndex.h"

/* Global variables */

#define NBTABLES 5
/* Get from :
 *   http://embryology.med.unsw.edu.au/DNA/Genetic_Codes2.htm
 * 0: std code
 * 1: Vertebrate Mitochondrial Code
 * 2: Yeast Mitochondrial Code
 * 3: Mold, Protozoan, and Coelenterate Mitochondrial Code 
 *    and the Mycoplasma/Spiroplasma Code
 * 4: Invertebrate Mitochondrial Code
 */

/** \var codingtable 
 * \brief This is a list of character string. Each character string
 * must be of size 65, i.e. one letter for each tri-nucleotides +
 * a sign used when a letter non nucleotide (N) is in a tri-nucleotides.
 */
char * codingtable [NBTABLES] = { \
"FFLLSSSSYY**CC*WLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSS**VVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWTTTTPPPPHHQQR--RIIMMTTTTNNKKSSRRVVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIIMTTTTNNKKSSRRVVVVAAAADDEEGGGG-", \
"FFLLSSSSYY**CCWWLLLLPPPPHHQQRRRRIIMMTTTTNNKKSSSSVVVVAAAADDEEGGGG-"};

extern "C"
{
  void strtranslateR (char ** nuc, char ** prot, int * quelle_table,
		      const char **  chars_table);

  SEXP print_code_table (SEXP quelle_tableR, SEXP chars_tableR);
  
  SEXP translateR (SEXP bufnoR,SEXP fromR, SEXP toR, SEXP strandR, SEXP QuelleTableR, SEXP chars_tableR);

}


/** \brief translate a charater string like "FFLLSSSSYY**CC*WLLLLPPPP..."
 * of size 65 characters to a human reading table like TTT -> F.
 *
 * Conversion table:
 * All in the order:
 * 0   1   2   3   4
 * TTT TTC TTA TTG TCT... 
 * F   F   L   L   S 
 * Letters in this order T=0,C=1,A=2,G=3
 * 
 * In R:
 *  .C("strnuctoprotR","TTTTTCTTATTGTCT","            ")
 */

void strtranslateR (char ** nuc, char ** prot, int * quelle_table,
		    const char **  chars_table)
{
  int i=0,j=0,k,nameprot;
  int longnuc;
  const char * table;
  extern char * codingtable [NBTABLES];

  longnuc = strlen(*nuc);

  /* Choose coding table */
  table = codingtable[0];
  if((quelle_table[0] > 0) && (quelle_table[0] <  NBTABLES) ) 
    table= codingtable[quelle_table[0]];

  if(strlen(*chars_table)==65)
    table = * chars_table;


  /* while( (*nuc) [i] && (*nuc) [i+1] && (*nuc) [i+2])*/
  while( i < longnuc-2)
    {

      nameprot = 0;

      for(k = 2;k>=0; k--)
	{
	  switch((*nuc)[i++])
	    {
	    case 'T': break;
	    case 'U': break;
	    case 't': break;
	    case 'u': break;
	    case 'C': nameprot = nameprot + (int)pow(4,k); break;
	    case 'c': nameprot = nameprot + (int)pow(4,k); break;
	    case 'A': nameprot = nameprot + 2*(int)pow(4,k); break;
	    case 'a': nameprot = nameprot + 2*(int)pow(4,k); break;
	    case 'G': nameprot = nameprot + 3*(int)pow(4,k); break;
	    case 'g': nameprot = nameprot + 3*(int)pow(4,k); break;
	    default:  nameprot = 64; 
	    }
	}
      if(nameprot > 64 ) nameprot = 64;
      (*prot)[j++] = table[nameprot];
    }
  (*prot)[j]= '\0';
}


SEXP print_code_table (SEXP quelle_tableR, SEXP chars_tableR)
{
  SEXP ans;
  int i,j,k,l;
  const char * table;
  int * quelle_tableC;
  const char * chars_tableC;
  char  nucl[5] = "UCAG";
  char  nucl_tmp[4];
  char  prot_tmp[2];
  extern char * codingtable [NBTABLES];

  /* Import from R input */
  quelle_tableC = INTEGER(quelle_tableR);
  chars_tableC = CHAR(STRING_ELT(chars_tableR,0));
	  
  /* Choose coding table */
  table = codingtable[0];
  if((quelle_tableC[0] > 0) && (quelle_tableC[0] <  NBTABLES) ) 
    table= codingtable[quelle_tableC[0]];

  if((strlen(chars_tableC)==65) || (strlen(chars_tableC)==64))
    table = chars_tableC;
  

  /* Allocation of result into R */
  PROTECT(ans = allocVector(STRSXP, 128));
  l=0;
  nucl_tmp[3]='\0';
  for(i =0; i<4; i++)
    {
      nucl_tmp[0]= nucl[i];
      for(j =0; j<4; j++)
	{
	  nucl_tmp[1]= nucl[j];
	  for(k =0; k<4; k++)
	    {
	      nucl_tmp[2]= nucl[k];
	      SET_STRING_ELT(ans, l++, mkChar(nucl_tmp));
	    }
	}
    }
  
  prot_tmp[1]='\0';
  for (i = 0 ; i<64; i++)
    {
      prot_tmp[0] = table[i];
      SET_STRING_ELT(ans, l++, mkChar(prot_tmp));
    }

  UNPROTECT(1);
  return(ans);
}


/** Function Translate
 * \brief translate nucleotide into protein
 */
SEXP translateR (SEXP bufnoR,SEXP fromR, SEXP toR, SEXP strandR, SEXP QuelleTableR, SEXP chars_tableR)
{
  SEXP ans;
  int * fromC;
  int * toC;
  int * strandC;
  int  bufnoC;
  int * quelle_tableC;
  char * proteine;
  char * nucleotides;
  char * Buffie;
  const char * chars_tableC;

  int nbregions,i,sizealloc,longbufno,onestrand=1,longueur;

  /* Get inputs from R */
  bufnoC = INTEGER(bufnoR)[0];
  fromC = INTEGER(fromR);
  toC = INTEGER(toR);
  strandC = INTEGER(strandR);
  quelle_tableC = INTEGER(QuelleTableR);
  nbregions =  LENGTH(fromR);
  chars_tableC = CHAR(STRING_ELT(chars_tableR,0));
  
  /* 2 cas of error */
  if((LENGTH(toR) != nbregions) )
    return(libIndex::returnInteger(-1));

  Buffie = myGeneRbuffer(bufnoC);
  if (Buffie == NULL)
    return(libIndex::returnInteger(-1));

  longbufno =   strlen(Buffie);

  /* allocation of temporary proteine */
  sizealloc = 0;
  for (i=0; i < nbregions; i++)
    {
      if(toC[i] == 0)
	toC[i] = longbufno;
      if((toC[i] - fromC[i] + 1) >sizealloc )
	sizealloc =toC[i] - fromC[i] + 1;
    }
  proteine = (char *) malloc( ((int) (sizealloc+2)/3 )* sizeof(char));
  nucleotides = (char *) malloc((sizealloc+1) * sizeof(char));
  
  /*  printf("Allocation faite nuc %d prot %d\n",sizealloc+1,((int) (sizealloc+2)/3 )); */

  if(LENGTH(strandR) ==nbregions)
    onestrand = nbregions+1;

   /* Allocation of result into R */
  PROTECT(ans = allocVector(STRSXP, nbregions));
  
  for (i=0; i < nbregions; i++)
    {
      Buffie = myGeneRbuffer(bufnoC,strandC[i%onestrand]);
    
      longueur = toC[i]-fromC[i]+1;
      if(longueur<0)
	longueur=1;
      nucleotides = strncpy(nucleotides,Buffie+fromC[i]-1,longueur);
      nucleotides[longueur] = '\0';
      /*  printf("Ds la boucle %d %d %s\n",strlen(nucleotides),fromC[i],nucleotides); */
  
      strtranslateR(&nucleotides,&proteine,quelle_tableC, &chars_tableC);

      /* Return to R */
      SET_STRING_ELT(ans, i, mkChar(proteine));
    }

  UNPROTECT(1);
  free(proteine);
  free(nucleotides);
  return(ans);
}
